from conda_forge_metadata.libcfgraph import (
    get_libcfgraph_artifact_data,
    get_libcfgraph_index,
    get_libcfgraph_pkgs_for_import,
)


def test_get_libcfgraph_index():
    lcfi = get_libcfgraph_index()
    assert len(lcfi) > 0
    assert isinstance(lcfi, list)
    assert isinstance(lcfi[0], str)
    assert lcfi[0].startswith("artifacts/")


def test_get_libcfgraph_artifact_data():
    data = get_libcfgraph_artifact_data(
        "conda-forge",
        "noarch",
        "flake8-6.0.0-pyhd8ed1ab_0.conda",
    )
    assert data is not None
    assert data["name"] == "flake8"
    assert data["version"] == "6.0.0"


def test_get_libcfgraph_artifact_data_none():
    data = get_libcfgraph_artifact_data(
        "conda-forge",
        "noarchhh",
        "flake8-6.0.0-pyhd8ed1ab_0.conda",
    )
    assert data is None


def test_get_libcfgraph_pkgs_for_import():
    pkgs, nm = get_libcfgraph_pkgs_for_import("numpy")
    assert pkgs is not None
    assert nm == "numpy"
    assert "numpy" in pkgs

    pkgs, nm = get_libcfgraph_pkgs_for_import("numpy.linalg")
    assert pkgs is not None
    assert nm == "numpy"
    assert "numpy" in pkgs

    # something bespoke
    pkgs, nm = get_libcfgraph_pkgs_for_import("eastlake")
    assert pkgs is not None
    assert nm == "eastlake"
    assert "des-eastlake" in pkgs
    assert len(pkgs) == 1

    pkgs, nm = get_libcfgraph_pkgs_for_import("scipy")
    assert pkgs is not None
    assert nm == "scipy"
    assert "scipy" in pkgs



from functools import lru_cache
from typing import Any, TypedDict

import requests

from conda_forge_metadata.types import CondaPackageName


class FeedstockOutputsConfig(TypedDict):
    outputs_path: str
    shard_level: int
    shard_fill: str


@lru_cache(maxsize=1)
def feedstock_outputs_config() -> FeedstockOutputsConfig:
    ref = "main"
    req = requests.get(
        "https://raw.githubusercontent.com/conda-forge/feedstock-outputs/"
        f"{ref}/config.json"
    )
    req.raise_for_status()
    return req.json()


def sharded_path(name: CondaPackageName) -> str:
    """
    Get the path to the sharded JSON path in the feedstock_outputs repository.

    Parameters
    ----------
    name : str
        The name of the package.

    Returns
    -------
    path : str
        The path to the sharded JSON file. Leading slash is omitted.
    """
    # See https://github.com/conda-forge/feedstock-outputs/
    #     blob/c35451f2fb8b7/scripts/shard_repo.py
    # for sharding details.
    config = feedstock_outputs_config()
    outputs_path = config["outputs_path"]
    shard_level = config["shard_level"]
    shard_fill = config["shard_fill"]

    name = name.lower()
    chars = [c for c in name if c.isalnum()][:shard_level]
    while len(chars) < shard_level:
        chars.append(shard_fill)

    return f"{outputs_path}/{'/'.join(chars)}/{name}.json"


@lru_cache(maxsize=1024)
def package_to_feedstock(name: CondaPackageName, **request_kwargs: Any) -> str:
    """Map a package name to the feedstock name(s).

    Parameters
    ----------
    package : str
        The name of the package.
    request_kwargs : dict
        Keyword arguments to pass to ``requests.get``.

    Returns
    -------
    feedstock : str
        The name of the feedstock, without the ``-feedstock`` suffix.
    """
    assert name, "name must not be empty"

    ref = "main"
    path = sharded_path(name)
    req = requests.get(
        f"https://raw.githubusercontent.com/conda-forge/feedstock-outputs/{ref}/{path}",
        **request_kwargs,
    )
    req.raise_for_status()
    return req.json()["feedstocks"]


if __name__ == "__main__":
    import sys

    if len(sys.argv) < 2:
        print("Usage: python -m conda_forge_metadata.feedstock_outputs <package>")
        sys.exit(1)

    for name in sys.argv[1:]:
        print(name, "->", package_to_feedstock(name))



from conda_forge_metadata.autotick_bot import get_pypi_name_mapping, map_pypi_to_conda


def test_map_pypi_to_conda():
    assert map_pypi_to_conda("numpy") == "numpy"
    assert map_pypi_to_conda("scipy") == "scipy"
    assert map_pypi_to_conda("21cmFAST") == "21cmfast"


def test_get_pypi_name_mapping():
    nmap = get_pypi_name_mapping()
    assert nmap is not None
    assert "conda_name" in nmap[0]



"""Commonly used type annotions for conda-forge-metadata."""

from typing import TypedDict

from typing_extensions import TypeAlias

CondaPackageName: TypeAlias = str
PypiPackageName: TypeAlias = str


class NameMappingEntry(TypedDict):
    pypi_name: PypiPackageName
    conda_name: CondaPackageName
    import_name: str
    mapping_source: str


class ArtifactData(TypedDict):
    """The data for a single artifact."""

    # the metadata version format
    metadata_version: int
    # the package name
    name: CondaPackageName
    # the package version
    version: str
    # the info/index.json file contents
    index: dict
    # the info/about.json file contents
    about: dict
    # the fully rendered recipe at either info/recipe/meta.yaml or info/meta.yaml as
    # a dict
    rendered_recipe: dict
    # the template recipe as a string from info/recipe/meta.yaml.template - could be
    # the rendered recipe as a string if no template was found
    raw_recipe: "str | None"
    # the conda_build_config.yaml used for building the recipe at
    # info/recipe/conda_build_config.yaml
    conda_build_config: dict
    # a list of files in the recipe from info/files with elements ending in .pyc or
    # .txt filtered out.
    files: "list[str]"



from __future__ import annotations

import typing
from functools import lru_cache

import requests
from ruamel import yaml

if typing.TYPE_CHECKING:
    from ..types import CondaPackageName, NameMappingEntry, PypiPackageName


@lru_cache(maxsize=1)
def get_pypi_name_mapping() -> list[NameMappingEntry]:
    req = requests.get(
        "https://raw.githubusercontent.com/regro/cf-graph-countyfair/"
        "master/mappings/pypi/name_mapping.yaml"
    )
    req.raise_for_status()
    return yaml.YAML(typ="safe").load(req.text)


@lru_cache(maxsize=1)
def get_grayskull_pypi_mapping() -> dict[PypiPackageName, NameMappingEntry]:
    req = requests.get(
        "https://raw.githubusercontent.com/regro/cf-graph-countyfair/"
        "master/mappings/pypi/grayskull_pypi_mapping.json"
    )
    req.raise_for_status()
    return req.json()


def map_pypi_to_conda(pypi_name: PypiPackageName) -> CondaPackageName:
    """Map a package's PyPi name to the most likely Conda name.

    Parameters
    ----------
    pypi_name : str
        The name on PyPi. This is case-sensitive.

    Returns
    -------
    conda_name : str
        The most likely Conda name.
    """
    pypi_map = get_grayskull_pypi_mapping()
    return pypi_map.get(pypi_name, {}).get("conda_name", pypi_name.lower())



from conda_forge_metadata.autotick_bot import map_import_to_package


def test_map_import_to_package():
    assert map_import_to_package("numpy") == "numpy"
    assert map_import_to_package("numpy.linalg") == "numpy"

    # something bespoke
    assert map_import_to_package("eastlake") == "des-eastlake"

    assert map_import_to_package("scipy") == "scipy"



"""
Use conda-package-streaming to fetch package metadata
"""
from contextlib import closing

from conda_package_streaming.package_streaming import stream_conda_component
from conda_package_streaming.url import conda_reader_for_url


def get_streamed_artifact_data(channel: str, subdir: str, artifact: str):
    if not channel.startswith("http"):
        if channel in ("pkgs/main", "pkgs/r", "pkgs/msys2"):
            channel = f"https//repo.anaconda.com/{channel}"
        else:
            channel = f"https://conda.anaconda.org/{channel}"

    # .conda artifacts can be streamed directly from an anaconda.org channel
    url = f"{channel}/{subdir}/{artifact}"
    filename, conda = conda_reader_for_url(url)

    with closing(conda):
        yield from stream_conda_component(filename, conda, component="info")
